* SQL-PREMIERS-PAS : Requtes SQL utilises dans le cadre de la prsentation de
                     Jean Hardy au Club SAS de Qubec - 1er mai 2014;

LIBNAME EDUC "C:\SEMINAIRES\CUSQ-SQL-PREMIERS-PAS\";

OPTIONS LOCALE="FR_CA";
PROC SQL;
* 3.1) Extraire toutes les variables;
  SELECT *
    FROM EDUC.PERS;

* 3.2) Extraire certaines variables et ranges; 
  SELECT MATRICUL, NOM, DEPT, SALAIRE
    FROM EDUC.PERS
    WHERE DEPT IN (10 20 51 66);

* 3.3) Cration de table ou de vues; 
  CREATE TABLE WORK.EXTRAIT AS
    SELECT MATRICUL, NOM, DEPT, SALAIRE
    FROM EDUC.PERS
    WHERE DEPT IN (10 20 51 66);
    
* 3.4) Tri des ranges; 
  CREATE TABLE WORK.EXTRAIT AS
    SELECT MATRICUL, NOM, DEPT, SALAIRE
    FROM EDUC.PERS
    WHERE DEPT IN (10 20 51 66)
    ORDER BY DEPT, SALAIRE DESCENDING;

* 4) Cration de variables;
* 4.1) Mot-cl AS et attribut de variables; 
  SELECT MATRICUL, NOM, COMM, SALAIRE, 
         COMM/SALAIRE AS RT_COMM FORMAT=NLPCT7.1
    FROM EDUC.PERS;
* 4.2) Usage des fonctions;
  SELECT CATX("-",SUBSTR(NOM,1,4),DEPT) 
             AS IDENT LENGTH=8,
         COMM FORMAT=NLMNY9., SALAIRE FORMAT=NLMNY9.,
         SUM(SALAIRE,COMM) AS REV FORMAT=NLMNY9.,
         COMM/CALCULATED REV AS RATIO FORMAT=NLNUM5.2 
    FROM EDUC.PERS;
* 4.3) Colonnes d'une expression n'ont pas besoin d'tre nommes dans SELECT;
  SELECT MATRICUL, NOM, DEPT, POSTE,
         SUM(SALAIRE,COMM) AS REV FORMAT=NLMNY9.
    FROM EDUC.PERS;
* 4.4) CASE / WHEN;
  SELECT MATRICUL, DEPT, COMM,
         CASE WHEN MISSING(COMM)    THEN "INCONNU"
              WHEN COMM<3000        THEN "- DE 3 000"
              WHEN 3000<=COMM<=6000 THEN "3 000  6000"
              ELSE                       "PLUS DE 6 000"
         END AS CAT_COMM  LENGTH=15
    FROM EDUC.PERS;

* 5) DISTINCT - un paramtre ... distinct;
* 5.1) Une variable;
  SELECT DISTINCT DEPT
    FROM EDUC.PERS;
* 5.2) Plusieurs variables;
  SELECT DISTINCT DEPT, POSTE 
    FROM EDUC.PERS;
QUIT;
* 5.3) quivalent avec PROC SORT;
PROC SORT  DATA=EDUC.PERS(KEEP=DEPT POSTE) 
           OUT=WORK.UNIQUES
           NODUPKEY;
  BY DEPT POSTE;
RUN;

* 6) Balayage vertical - fonctions verticales tel SUM(var)
     ou MEAN(var) ou COUNT(var) ou COUNT(DISTINCT var);
PROC SQL;
* 6.1) Rsultat du balayage joint au dtail de chaque ligne;
  SELECT MATRICUL, DEPT, SALAIRE, MEAN(SALAIRE) AS MOY,
         SALAIRE-(CALCULATED MOY) AS ECART FORMAT=NLNUM6.
    FROM EDUC.PERS;
* 6.2) SELECT ne contient que des fonctions de balayage vertical;
  SELECT MEAN(SALAIRE) AS SAL_MOYEN,
         SUM(SALAIRE) AS SAL_SOMME,
         SUM(COMM) AS COMM_SOMME,
         MAX(ANNEES) AS ANN_MAX,
         NMISS(COMM) AS SANS_COMM
    FROM EDUC.PERS;
* 6.3) COUNT et ses usages;
  SELECT COUNT(*) AS NB_RANGEES,
         COUNT(COMM) AS NB_COMM_VALIDES,
         COUNT(DISTINCT COMM) AS NB_COMM_DIST
    FROM EDUC.PERS;
* 6.4) MAUVAIS usages de COUNT sur une expression;
  SELECT COUNT(COMM>=5000) AS NB_COMM_5000
    FROM EDUC.PERS;
* 6.5) Usages corrects pour compter le nombre de valeurs
       vraies retournes par une expression;
  SELECT SUM(COMM>=5000) AS NB_COMM_5000,
         SUM(POSTE IN ("CADRE" "ADMIN")) AS NB_GESTION
    FROM EDUC.PERS;

* 7) AGRGATIONS;
* 7.1) Agrgation de base;
  SELECT DEPT,
         MEAN(SALAIRE) AS MOY  FORMAT=NLMNY9.,
         COUNT(SALAIRE) AS NB_SAL,
         COUNT(COMM) AS NB_COMM
    FROM EDUC.PERS
    GROUP BY DEPT;
* 7.2) Ajout par erreur d'une variable trangre  l'agrgation et au balayage;
  SELECT DEPT, NOM, SALAIRE FORMAT=NLMNY9.,
         MEAN(SALAIRE) AS MOY  FORMAT=NLMNY9.
    FROM EDUC.PERS
    GROUP BY DEPT;
* 7.3) Utiliser le phnomne prcdent pour calculer l'cart entre chaque employ
       et le salaire moyen des employs de son dpartement;
  SELECT DEPT, NOM, SALAIRE,
         MEAN(SALAIRE) AS MOY,
         SALAIRE-(CALCULATED MOY) AS ECART FORMAT=NLMNY9.
    FROM EDUC.PERS
    GROUP BY DEPT;
* 7.4) Filtrage sur une statistique agrge;
  SELECT DEPT,
         MEAN(SALAIRE) AS MOY  FORMAT=NLMNY9.,
         COUNT(SALAIRE) AS NB_SAL
    FROM EDUC.PERS
    GROUP BY DEPT
    HAVING MOY >= 8500;
QUIT;
* 8) JOINTURES;
* 8.0) Jointure interne via DATA;
PROC SORT  DATA=EDUC.CHILDREN  OUT=WORK.CHILDREN;
  BY FAM_NUM;
RUN;
PROC SORT  DATA=EDUC.FAMILY  OUT=WORK.FAMILY;
  BY FAM_NUM;
RUN;
DATA JOINT_INT;
  MERGE WORK.CHILDREN(IN=C)
        WORK.FAMILY(IN=F);
  BY FAM_NUM;
  IF C=1 AND F=1;
RUN;
* 8.1) Jointure interne dtaille;
PROC SQL;
  SELECT C.FAM_NUM, CHLD_AGE, CHLD_SEX,
         PROV, CITY
    FROM EDUC.CHILDREN AS C
         INNER JOIN
         EDUC.FAMILY AS F
         ON C.FAM_NUM=F.FAM_NUM;
* 8.2) Jointure interne abrge;
  SELECT C.*, PROV, CITY
    FROM EDUC.CHILDREN AS C
         INNER JOIN
         EDUC.FAMILY AS F
         ON C.FAM_NUM=F.FAM_NUM;
* 8.3) Jointure gauche;
  SELECT C.*, PROV, CITY
    FROM EDUC.CHILDREN AS C
         LEFT JOIN
         EDUC.FAMILY AS F
         ON C.FAM_NUM=F.FAM_NUM;
* 8.4) Fusion sans variable de mme nom dans les deux tables;
  CREATE TABLE WORK.FAMILY AS
    SELECT FAM_NUM AS ID_CODE, PROV, CITY
    FROM EDUC.FAMILY;
  SELECT C.*, PROV, CITY
    FROM EDUC.CHILDREN AS C
         LEFT JOIN
         WORK.FAMILY AS F
         ON C.FAM_NUM=F.ID_CODE;


PROC SQL;
* 9) Requte de type "SOUS-SELECT" - NOUVELLE FORMULE;
* 9A) Requte conventionnelle;
  SELECT *
  FROM EDUC.CHILDREN
  WHERE FAM_NUM IN (101 102 103 108) 
  ORDER BY FAM_NUM;
* 9B) Sous-requte;
  SELECT *
  FROM EDUC.CHILDREN
  WHERE FAM_NUM IN (SELECT DISTINCT FAM_NUM
                    FROM EDUC.FAMILY
                    WHERE PROV="ON")
  ORDER BY FAM_NUM;
* 9C) Via une jointure interne;
  SELECT C.*
  FROM EDUC.CHILDREN AS C
         INNER JOIN
       EDUC.FAMILY AS F
         ON C.FAM_NUM=F.FAM_NUM AND
            F.PROV = "ON"
  ORDER BY FAM_NUM;
QUIT;

